@@ -10,7 +10,8 @@ from typing import (Callable,
                     MutableSequence,
                     Optional)
 
-from .hints import (Domain,
+from .hints import (Comparator,
+                    Domain,
                     Key)
 
 
@@ -38,7 +39,7 @@ def nth_largest(sequence: MutableSequence[Domain],
         in the sequence sorted by key in descending order
         (e.g. ``n = 0`` corresponds to the maximum element)
     :param key:
-        a single argument ordering function,
+        single argument ordering function,
         if none is specified compares elements themselves
     :returns: n-th largest element of the sequence
 
@@ -60,7 +61,9 @@ def nth_largest(sequence: MutableSequence[Domain],
     >>> nth_largest(sequence, 20, key=abs)
     0
     """
-    return _select(sequence, n, 0, len(sequence) - 1, key, gt)
+    return select(sequence, n,
+                  key=key,
+                  comparator=gt)
 
 
 def nth_smallest(sequence: MutableSequence[Domain],
@@ -87,7 +90,7 @@ def nth_smallest(sequence: MutableSequence[Domain],
         in the sequence sorted by key in ascending order
         (e.g. ``n = 0`` corresponds to the minimum element)
     :param key:
-        a single argument ordering function,
+        single argument ordering function,
         if none is specified compares elements themselves
     :returns: n-th smallest element of the sequence
 
@@ -109,15 +112,62 @@ def nth_smallest(sequence: MutableSequence[Domain],
     >>> nth_smallest(sequence, 20, key=abs)
     10
     """
-    return _select(sequence, n, 0, len(sequence) - 1, key, lt)
+    return select(sequence, n,
+                  key=key,
+                  comparator=lt)
 
 
-def _select(sequence: MutableSequence[Domain],
-            n: int,
-            start: int,
-            stop: int,
-            key: Optional[Key],
-            comparator: Callable[[Domain, Domain], bool]):
+def select(sequence: MutableSequence[Domain],
+           n: int,
+           *,
+           start: int = 0,
+           stop: Optional[int] = None,
+           key: Optional[Key] = None,
+           comparator: Comparator) -> Domain:
+    """
+    Partially sorts given sequence and returns n-th element.
+
+    +------------+-------------+-----------------+------------------+
+    | complexity |    best     |     average     |      worst       |
+    +------------+-------------+-----------------+------------------+
+    |    time    | ``O(size)`` |   ``O(size)``   | ``O(size ** 2)`` |
+    +------------+-------------+-----------------+------------------+
+    |   memory   |  ``O(1)``   | ``O(log size)`` |   ``O(size)``    |
+    +------------+-------------+-----------------+------------------+
+
+    where ``size = len(sequence)``.
+
+    :param sequence: sequence to select from
+    :param n: index of the element to select
+    :param start: index to start selection from
+    :param stop: index to stop selection at
+    :param key:
+        single argument ordering function,
+        if none is specified compares elements themselves
+    :param comparator:
+        binary predicate that defines the sorting order
+    :returns:
+        n-th element of the sequence
+        with slice partially sorted by key in given order
+
+
+    >>> from operator import gt, lt
+    >>> sequence = list(range(-10, 11))
+    >>> select(sequence, 0, stop=5, comparator=gt)
+    -5
+    >>> select(sequence, 0, stop=5, comparator=lt)
+    -10
+    >>> select(sequence, 20, start=15, comparator=lt)
+    10
+    >>> select(sequence, 20, start=15, comparator=gt)
+    5
+    >>> select(sequence, 5, start=5, stop=15, key=abs, comparator=lt)
+    0
+    >>> select(sequence, 5, start=5, stop=15, key=abs, comparator=gt)
+    10
+    """
+    if stop is None:
+        stop = len(sequence) - 1
     while True:
         pivot_index = _partition(sequence, start, stop, key, comparator)
         if pivot_index < n:
